<?php

/**
 * @file
 * Contains \Drupal\custom_pub\Form\CustomPublishingOptionForm.
 */

namespace Drupal\custom_pub\Form;

use Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\node\Entity\NodeType;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class CustomPublishingOptionForm.
 *
 * @package Drupal\custom_pub\Form
 */
class CustomPublishingOptionForm extends EntityForm {
  /**
   * @var \Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface
   */
  protected $entityDefinitionManager;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * CustomPublishingOptionForm constructor.
   * @param \Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface $entity_definition_manager
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_type_manager
   */
  public function __construct(EntityDefinitionUpdateManagerInterface $entity_definition_manager, EntityTypeManager $entity_type_manager) {
    $this->entityDefinitionManager = $entity_definition_manager;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $entity_definition_manager = $container->get('entity.definition_update_manager');
    $entity_type_manager = $container->get('entity_type.manager');
    return new static($entity_definition_manager, $entity_type_manager);
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $entity = $this->entity;

    $types = [];

    foreach (NodeType::loadMultiple() as $id => $type) {
      $types[$type->id()] = $type->label();
    }

    $form['label'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $entity->label(),
      '#description' => $this->t("The label for this publishing option."),
      '#required' => TRUE,
    );

    $form['id'] = array(
      '#type' => 'machine_name',
      '#default_value' => $entity->id(),
      '#machine_name' => array(
        'exists' => '\Drupal\custom_pub\Entity\CustomPublishingOption::load',
      ),
      '#disabled' => !$entity->isNew(),
    );

    $form['description'] = array(
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#maxlength' => 255,
      '#default_value' => $entity->getDescription(),
      '#description' => $this->t("Add a description for this publishing option."),
      '#required' => TRUE,
    );

    $form['types'] = array(
      '#type' => 'checkboxes',
      '#title' => $this->t('Node Types'),
      '#default_value' => $entity->getNodeTypes(),
      '#description' => $this->t('Select the node types you want this option to be available on.'),
      '#options' => $types,
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // check to see if a field definition exists by this machine name to prevent collisions/overwrites
    $entity = $this->entity;
    $storage_definition = $this->entityDefinitionManager->getFieldStorageDefinition($entity->id(), 'node');

    if ($entity->isNew() && isset($storage_definition)) {
      $form_state->setError($form['id'], $this->t('Cannot use machine name %name - field definition already exists.', ['%name' => $entity->id()]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $entity = $this->entity;

    $is_new = $entity->isNew();
    $id = $entity->id();
    $label = $entity->label();
    $description = $entity->getDescription();

    $status = $entity->save();

    switch ($status) {
      case SAVED_NEW:
        drupal_set_message($this->t('Created the %label custom publishing option.', [
          '%label' => $label,
        ]));
        break;

      default:
        drupal_set_message($this->t('Saved the %label custom publishing option.', [
          '%label' => $label,
        ]));
    }

    // @todo: attempt to auto create Actions to set/unset bool value if possible
    // @todo: possibly create small methods for the following code actions

    if ($is_new) {
      // create the field definition for the node
      $storage_definition = BaseFieldDefinition::create('boolean')
        ->setLabel(t('@label', ['@label' => $label]))
        ->setDescription(t('@description', ['@description' => $description]))
        ->setRevisionable(TRUE)
        ->setTranslatable(TRUE)
        ->setDefaultValue(FALSE)
        ->setDisplayConfigurable('form', TRUE)
        ->setDisplayOptions(
          'form',
          [
            'type' => 'boolean_checkbox',
            'settings' => [
              'display_label' => TRUE,
            ],
            'weight' => 20,
          ]
        );

      $this->entityDefinitionManager->installFieldStorageDefinition($id, 'node', 'custom_pub', $storage_definition);
    } else {
      $storage_definition = $this->entityDefinitionManager->getFieldStorageDefinition($id, 'node');

      if (isset($storage_definition)) {
        // update the label and description on the definition
        $storage_definition
          ->setLabel(t('@label', ['@label' => $label]))
          ->setDescription(t('@description', ['@description' => $description]));

        $this->entityDefinitionManager->updateFieldStorageDefinition($storage_definition);
      }
    }

    $form_state->setRedirectUrl($entity->urlInfo('collection'));
  }

}
